/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.action;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.ApplicationCallback;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Action Klasse zum L&ouml;schen von Ordnern
 *
 * @author Frank Mnster
 *
 */
public class OrdnerDeleteAction implements Action {

    /**
     * Callback f&uunl; Benutzernachfragen
     */
    private static final ApplicationCallback CALLBACK = Application.getCallback();

    /**
     * Standart Konstruktor
     */
    public OrdnerDeleteAction() {
	// intentionally left blank.
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException { // NOPMD

	if (context == null || !(context instanceof Ordner) && !(context instanceof Ordner[])) {
	    throw new ApplicationException(I18N.tr("OrdnerDeleteAction.chooseFolderMsg")); //$NON-NLS-1$
	}

	if (context instanceof Ordner) {

	    handleDeleteOrdner((Ordner) context);

	} else {
	    handleDeleteOrdner((Ordner[]) context);
	}

    }

    private void deleteOrdner(final Ordner ordner) throws RemoteException, ApplicationException {
	final DBIterator<Register> register = ordner.getInhaltsverzeichniss();
	while (register.hasNext()) {
	    final Register registerEintrag = register.next();
	    registerEintrag.delete();
	}
	ordner.delete();
    }

    private void handleDeleteOrdner(final Ordner ordner) throws ApplicationException {

	// before deleting the folder, we show up a confirm dialog ;)
	final String question = I18N.tr("OrdnerDeleteAction.folderSureMsg"); //$NON-NLS-1$
	try {
	    if (OrdnerDeleteAction.CALLBACK.askUser(question)) {
		deleteOrdner(ordner);
		Application.getMessagingFactory().sendMessage(new StatusBarMessage(
			I18N.tr("OrdnerDeleteAction.folderSuccessMsg"), StatusBarMessage.TYPE_SUCCESS)); //$NON-NLS-1$
	    }
	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("OrdnerDeleteAction.FolderErrorMSg"), e); //$NON-NLS-1$
	    throw new ApplicationException(I18N.tr("OrdnerDeleteAction.FolderErrorMSg"), e); //$NON-NLS-1$
	}
    }

    private void handleDeleteOrdner(final Ordner[] ordnerArray) // NOPMD
	    throws ApplicationException {
	// before deleting the folder, we show up a confirm dialog ;)
	final String question = I18N.tr("OrdnerDeleteAction.foldersSureMsg"); //$NON-NLS-1$
	try {
	    if (OrdnerDeleteAction.CALLBACK.askUser(question)) {
		for (final Ordner ordner : ordnerArray) {
		    deleteOrdner(ordner);
		}
		Application.getMessagingFactory().sendMessage(new StatusBarMessage(
			I18N.tr("OrdnerDeleteAction.foldersSuccessMsg"), StatusBarMessage.TYPE_SUCCESS)); //$NON-NLS-1$
	    }
	} catch (final Exception e) { // NOPMD
	    Logger.error(I18N.tr("OrdnerDeleteAction.foldersErrorMsg"), e); //$NON-NLS-1$
	    throw new ApplicationException(I18N.tr("OrdnerDeleteAction.foldersErrorMsg"), e); //$NON-NLS-1$
	}
    }
}
