/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.gui.controller;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.FontDialog;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.db.server.RegisterImpl;
import de.frankmuenster.jameica.orverw.gui.dialog.PrintDialog;
import de.frankmuenster.jameica.orverw.gui.input.ToolTipButton;
import de.frankmuenster.jameica.orverw.gui.menu.RegisterTableMenu;
import de.frankmuenster.jameica.orverw.gui.views.OrdnerListView;
import de.frankmuenster.jameica.orverw.parts.RegisterTablePart;
import de.frankmuenster.jameica.orverw.parts.RegisterTypEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPart;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartButtonObject;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartListener;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.LabelSizeEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsButtonObject;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsListener;
import de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsPart;
import de.frankmuenster.jameica.orverw.print.PrintRegister;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.TableChangeListener;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Controller for the edit folder view
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.ExcessiveImports", "PMD.CyclomaticComplexity", "PMD.StdCyclomaticComplexity",
        "PMD.TooManyMethods" })
public class OrdnerBearbeitenController extends AbstractControl
        implements LabelPartListener, TableChangeListener, RegisterSettingsListener {

    /**
     * Unser Ordner
     */
    private transient Ordner ordner;

    /**
     * und die Registereintr&auml;ge
     */
    private transient DBIterator<Register> register;

    /**
     * Ist es ein neuer Ordner
     */
    private transient boolean neuerOrdner;

    /**
     * Die Gr&ouml;&szlig;e des Labels
     */
    private transient LabelSizeEnum labelSize;

    /**
     * Das Part f&uuml;r den Ordnerr&uuml;cken
     */
    private transient LabelPart ordnerRuecken;

    /**
     * Das Part f&uuml;r die Registereintr&auml;ge
     */
    private transient RegisterTablePart registerTable;

    /**
     * Das Part f&uuml; die Regsitereinstellungen
     */
    private transient RegisterSettingsPart registerSettings;

    // Buttons
    /**
     * Unser Speicher-Button
     */
    private transient Button saveButton;

    /**
     * Unser Abbrech-Button
     */
    private transient Button cancelEditButton;

    /**
     * Die Buttons f&uuml; die unterschiedlichen Label-Gr&ouml;&szlig;en
     */
    private transient List<ToolTipButton> registerSizeBtns;

    /**
     * Der Druck-Button
     */
    private transient Button printButton;

    /**
     * Zeigt an, ob sich der Inhalt ver&auml;ndert hat.
     */
    private transient boolean inhaltChanged;

    /**
     * Unser Konstruktor f&uuml;r den Knotroller
     *
     * @param view
     * @throws RemoteException
     */
    public OrdnerBearbeitenController(final AbstractView view) throws RemoteException {

        super(view);
        final Object viewObject = view.getCurrentObject();
        if (viewObject instanceof LabelSizeEnum) {
            this.neuerOrdner = true;
            this.labelSize = (LabelSizeEnum) viewObject;
            final DBService service = Settings.getDBService();
            this.ordner = (Ordner) service.createObject(Ordner.class, null);
            this.ordner.setWidth(this.labelSize.getWidth());
            this.ordner.loadDefaults();
        } else if (viewObject instanceof Ordner) {
            this.neuerOrdner = false;
            this.ordner = (Ordner) this.getCurrentObject();
            this.register = this.ordner.getInhaltsverzeichniss();
            final Integer width = this.ordner.getWidth();
            this.labelSize = LabelSizeEnum.getLabelSize(width);
        }
        this.inhaltChanged = false;
    }

    /**
     * @return den Abbruch-Button cancle
     */
    public Button getCancelEditButton() {

        if (this.cancelEditButton != null) {
            return this.cancelEditButton; // NOPMD
        }
        this.cancelEditButton = new Button(I18N.tr("_cancel"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                OrdnerBearbeitenController.this.handleCancelEdit();
            }
        });
        return this.cancelEditButton;
    }

    /**
     * @return the size of the label as string
     */
    public String getLabelSize() {

        if (this.labelSize != null) {
            return this.labelSize.getValue(); // NOPMD
        }
        return I18N.tr("OrdnerBearbeitenController.NotDefined"); //$NON-NLS-1$
    }

    /**
     * @return das LabelPart das den Ordnerr&uuml;cken darstellt
     * @throws ApplicationException
     */
    public LabelPart getOrdnerRuecken() throws ApplicationException {

        if (this.ordnerRuecken != null) {
            return this.ordnerRuecken; // NOPMD
        }
        this.ordnerRuecken = new LabelPart(this.labelSize, this);
        try {
            this.ordnerRuecken.setTitelText(this.ordner.getTitel());
            this.ordnerRuecken.setFont(LabelPartSectionsEnum.TITLE, this.ordner.getTitelFont());
            this.ordnerRuecken.setForegroundColor(LabelPartSectionsEnum.TITLE, this.ordner.getTitelColor());
            this.ordnerRuecken.setAlignment(LabelPartSectionsEnum.TITLE, this.ordner.getTitelAlignment());
            this.ordnerRuecken.setContentText(this.ordner.getContent());
            this.ordnerRuecken.setFont(LabelPartSectionsEnum.CONTENT, this.ordner.getContentFont());
            this.ordnerRuecken.setForegroundColor(LabelPartSectionsEnum.CONTENT, this.ordner.getContentColor());
            this.ordnerRuecken.setAlignment(LabelPartSectionsEnum.CONTENT, this.ordner.getContentAlignment());
            this.ordnerRuecken.setFooterText(this.ordner.getFooter());
            this.ordnerRuecken.setFont(LabelPartSectionsEnum.FOOTER, this.ordner.getFooterFont());
            this.ordnerRuecken.setForegroundColor(LabelPartSectionsEnum.FOOTER, this.ordner.getFooterColor());
            this.ordnerRuecken.setAlignment(LabelPartSectionsEnum.FOOTER, this.ordner.getFooterAlignment());
            this.ordnerRuecken.setIdText(this.ordner.getID());
            this.ordnerRuecken.setIdVisible(this.ordner.isIdVisible());
            this.ordnerRuecken.setBgColour(this.ordner.getLabelBackground());
            this.ordnerRuecken.setBarColour(this.ordner.getBarColor());
            this.ordnerRuecken.setBarVisible(this.ordner.isBarVisible());
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("OrdnerBearbeitenController.setLabelValuesFailed"), e); //$NON-NLS-1$
        }
        return this.ordnerRuecken;
    }

    /**
     * Liefert den Print Button
     *
     * @return Button f&uur das Drucken
     */
    public Button getPrintButton() {

        if (this.printButton != null) {
            return this.printButton; // NOPMD
        }
        this.printButton = new Button(I18N.tr("_print"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                final List<Ordner> ordnerList = new ArrayList<>();
                ordnerList.add(OrdnerBearbeitenController.this.ordner);
                try {
                    final PrintDialog printDialog = new PrintDialog(ordnerList);
                    printDialog.open();
                } catch (final OperationCanceledException e) { // NOPMD
                    // Hier machen wir nix :-)
                } catch (final Exception e) { // NOPMD
                    throw new ApplicationException(I18N.tr("OrdnerBearbeitenController.OpenPrintDialogFailed"), e); //$NON-NLS-1$
                }
            }
        });
        if (this.isNeuerOrdner()) {
            this.printButton.setEnabled(false);
        }
        return this.printButton;
    }

    /**
     * Liefert eine Liste mit Buttons, um das Register neu zu bef&uuml;llen. Die List wird aus den RegisterTypeEnem
     * Werten erstellt.
     *
     * @return ein Liste mit Buttons, um das Register neu zu befllen.
     */
    public List<ToolTipButton> getRegisterButtons() {

        if (this.registerSizeBtns != null) {
            return this.registerSizeBtns; // NOPMD
        }
        this.registerSizeBtns = new ArrayList<>();
        for (final RegisterTypEnum registerType : RegisterTypEnum.values()) {
            @SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
            final ToolTipButton button = new ToolTipButton(registerType.getTitel(), new Action() {

                /**
                 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    try {
                        OrdnerBearbeitenController.this.checkRegisterSize(registerType);
                    } catch (final RemoteException e) {
                        throw new ApplicationException(I18N.tr("OrdnerBearbeitenController.createRegisterFailed"), e); //$NON-NLS-1$
                    }
                }
            });
            button.setToolTip(registerType.getTooltip());
            this.registerSizeBtns.add(button);
        }
        return this.registerSizeBtns;
    }

    /**
     * @return Das Einstellpanel f&uuml;r das Register
     * @throws ApplicationException
     */
    public RegisterSettingsPart getRegisterSettingsPart() throws ApplicationException {

        if (this.registerSettings != null) {
            return this.registerSettings; // NOPMD
        }
        this.registerSettings = new RegisterSettingsPart(this);
        try {
            this.registerSettings.setIndexFont(this.ordner.getRegisterIndexFont());
            this.registerSettings.setIndexBgColor(this.ordner.getRegisterIndexBackground());
            this.registerSettings.setContentFont(this.ordner.getRegisterContentFont());
            this.registerSettings.setContentBgColor(this.ordner.getRegisterContentBackground());
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("OrdnerBearbeitenController.setRegisterValuesFailed"), e); //$NON-NLS-1$
        }
        return this.registerSettings;
    }

    /**
     * @return die Registertabelle
     * @throws RemoteException
     */
    public RegisterTablePart getRegisterTabelle() throws RemoteException {

        if (this.registerTable != null) {
            return this.registerTable; // NOPMD
        }
        this.registerTable = new RegisterTablePart(this.register, null);
        this.registerTable.setContextMenu(new RegisterTableMenu(this));
        this.registerTable.addChangeListener(this);
        return this.registerTable;
    }

    /**
     * @return den Speicher-Button
     */
    public Button getSaveButton() {

        if (this.saveButton != null) {
            return this.saveButton; // NOPMD
        }
        this.saveButton = new Button(I18N.tr("_save"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                OrdnerBearbeitenController.this.handleSave();
            }
        });
        this.saveButton.setEnabled(false);
        return this.saveButton;
    }

    /**
     * @return <i>true</i> wenn es ein neuer Ordner ist, <i>false</i> sonst.
     */
    public boolean isNeuerOrdner() {

        return this.neuerOrdner;
    }

    /**
     * @see de.willuhn.jameica.gui.parts.TableChangeListener#itemChanged(java.lang .Object, java.lang.String,
     *      java.lang.String)
     */
    @Override
    public void itemChanged(final Object object, final String attribute, final String newValue)
            throws ApplicationException {

        if (object instanceof Register) {
            final Register eintrag = (Register) object;
            try {
                if (Logger.isLogging(Level.DEBUG)) {
                    Logger.debug(eintrag.getBeschreibung() + " - " + newValue); //$NON-NLS-1$
                }
                switch (attribute) {
                    case "idx": //$NON-NLS-1$
                        eintrag.setIndex(newValue);
                        break;
                    case "content": //$NON-NLS-1$
                        // Check length
                        this.checkLength(newValue);
                        eintrag.setBeschreibung(newValue);
                        break;
                    default:
                        break;
                }
                if (!this.inhaltChanged) {
                    this.inhaltChanged = true;
                    this.updateActionButtons();
                }
            } catch (final RemoteException e) {
                throw new ApplicationException(I18N.tr("OrdnerBearbeitenController.updateRegisterEntryFailed"), e); //$NON-NLS-1$
            }
        }
    }

    /**
     * @see LabelPartListener#notifyButtonClicked(LabelPartButtonObject)
     */
    @Override
    public void notifyButtonClicked(final LabelPartButtonObject buttonObject) {

        Boolean checkChange = true;
        final LabelPartButtonsEnum button = buttonObject.getButton();
        ColorDialog colorDialog;
        RGB colorRgb;
        Color color;
        switch (button) {
            case ALIGN_CENTER:
                this.ordnerRuecken.setAlignment(buttonObject.getSection(), SWT.CENTER);
                break;
            case ALIGN_LEFT:
                this.ordnerRuecken.setAlignment(buttonObject.getSection(), SWT.LEFT);
                break;
            case ALIGN_RIGHT:
                this.ordnerRuecken.setAlignment(buttonObject.getSection(), SWT.RIGHT);
                break;
            case FONT:
                final FontDialog fontDialog = new FontDialog(GUI.getShell());
                fontDialog.setEffectsVisible(false);
                fontDialog.setFontList(buttonObject.getFont().getFontData());
                final FontData fontData = fontDialog.open();
                if (fontData != null) {
                    final Font font = new Font(GUI.getDisplay(), fontData);
                    this.ordnerRuecken.setFont(buttonObject.getSection(), font);
                }
                break;
            case COLOUR:
                colorDialog = new ColorDialog(GUI.getShell());
                colorDialog.setRGB(buttonObject.getColour().getRGB());
                colorRgb = colorDialog.open();
                if (colorRgb != null) {
                    color = new Color(GUI.getDisplay(), colorRgb);
                    this.ordnerRuecken.setForegroundColor(buttonObject.getSection(), color);
                }
                break;
            case BACKGROUND_COLOUR:
                colorDialog = new ColorDialog(GUI.getShell());
                colorDialog.setRGB(buttonObject.getColour().getRGB());
                colorRgb = colorDialog.open();
                if (colorRgb != null) {
                    color = new Color(GUI.getDisplay(), colorRgb);
                    this.ordnerRuecken.setBgColour(color);
                }
                break;
            case BAR_COLOUR:
                colorDialog = new ColorDialog(GUI.getShell());
                colorDialog.setRGB(buttonObject.getColour().getRGB());
                colorRgb = colorDialog.open();
                if (colorRgb != null) {
                    color = new Color(GUI.getDisplay(), colorRgb);
                    this.ordnerRuecken.setBarColour(color);
                }
                break;
            case BAR_VISIBLE:
                this.ordnerRuecken.setBarVisible(!this.ordnerRuecken.getBarVisible());
                break;
            case ID_VISIBLE:
                this.ordnerRuecken.setIdVisible(!this.ordnerRuecken.isIdVisible());
                break;
            default:
                checkChange = false;
                break;
        }
        if (checkChange && !this.inhaltChanged) {
            this.inhaltChanged = true;
            this.updateActionButtons();
        }
    }

    /**
     * @see de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsListener#notifyButtonClicked
     *      (de.frankmuenster.jameica.orverw.parts.register.RegisterSettingsButtonObject)
     */
    @Override
    public void notifyButtonClicked(final RegisterSettingsButtonObject buttonObject) {

        boolean setChange = true;
        RegisterSettingsButtonsEnum button;
        button = buttonObject.getButton();
        FontDialog fontDialog;
        FontData fontData;
        ColorDialog colorDialog;
        RGB colorRgb;
        Color color;
        Font font;
        switch (button) {
            case INDEX_FONT:
                fontDialog = new FontDialog(GUI.getShell());
                fontDialog.setEffectsVisible(false);
                fontDialog.setFontList(buttonObject.getFont().getFontData());
                fontData = fontDialog.open();
                if (fontData != null) {
                    font = new Font(GUI.getDisplay(), fontData);
                    this.registerSettings.setIndexFont(font);
                }
                break;
            case INDEX_BACKGROUND_COLOR:
                colorDialog = new ColorDialog(GUI.getShell());
                colorDialog.setRGB(buttonObject.getColor().getRGB());
                colorRgb = colorDialog.open();
                if (colorRgb != null) {
                    color = new Color(GUI.getDisplay(), colorRgb);
                    this.registerSettings.setIndexBgColor(color);
                }
                break;
            case CONTENT_FONT:
                fontDialog = new FontDialog(GUI.getShell());
                fontDialog.setEffectsVisible(false);
                fontDialog.setFontList(buttonObject.getFont().getFontData());
                fontData = fontDialog.open();
                if (fontData != null) {
                    font = new Font(GUI.getDisplay(), fontData);
                    this.registerSettings.setContentFont(font);
                }
                break;
            case CONTENT_BACKGROUND_COLOR:
                colorDialog = new ColorDialog(GUI.getShell());
                colorDialog.setRGB(buttonObject.getColor().getRGB());
                colorRgb = colorDialog.open();
                if (colorRgb != null) {
                    color = new Color(GUI.getDisplay(), colorRgb);
                    this.registerSettings.setContentBgColor(color);
                }
                break;
            default:
                setChange = false;
                break;
        }
        if (setChange && !this.inhaltChanged) {
            this.inhaltChanged = true;
            this.updateActionButtons();
        }
    }

    /**
     * @see de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartListener#notifyLabelTextChanged()
     */
    @Override
    public void notifyLabelTextChanged() {

        if (!this.inhaltChanged) {
            this.inhaltChanged = true;
            this.updateActionButtons();
        }
    }

    /**
     * Checks the width of the new text, if it fits with the given Font into the register
     *
     * @param newText
     */
    private void checkLength(final String newText) {

        if (this.registerSettings.getContentFont() != null) {
            final GC graphic = new GC(GUI.getShell());
            graphic.setFont(this.registerSettings.getContentFont());
            if (graphic.textExtent(newText).x > PrintRegister.DESCRIPTION_WIDTH) {
                final StatusBarMessage msg = new StatusBarMessage(
                        I18N.tr("OrdnerBearbeitenController.registerEntryTooLong"), //$NON-NLS-1$
                        StatusBarMessage.TYPE_INFO);
                Application.getMessagingFactory().sendMessage(msg);
            }
            graphic.dispose();
        }
    }

    private void checkRegisterSize(final int rows, final RegisterTypEnum typ) throws RemoteException {

        String[] index;
        if (typ == null) {
            index = new String[rows];
            Arrays.fill(index, StringUtils.EMPTY);
        } else {
            index = typ.getIndex();
        }
        @SuppressWarnings("unchecked")
        final List<Register> items = this.registerTable.getItems();
        this.registerTable.removeAll();
        final DBService service = Settings.getDBService();
        if (rows < items.size()) {
            for (int i = items.size() - 1; i >= rows; i--) {
                items.remove(i);
            }
        }
        for (int i = 0; i < rows; i++) {
            Register reg;
            if (i < items.size()) {
                reg = items.get(i);
                reg.setIndex(index[i]);
            } else {
                reg = (RegisterImpl) service.createObject(Register.class, null);
                reg.setIndex(index[i]);
                reg.setBeschreibung(I18N.tr("OrdnerBearbeitenController.empty")); //$NON-NLS-1$
                reg.setOrdner(this.ordner);
                items.add(reg);
            }
            Logger.debug("RegBe: " + reg.getBeschreibung()); //$NON-NLS-1$
        }
        for (final Register reg1 : items) {
            this.registerTable.addItem(reg1);
        }
        if (!this.inhaltChanged) {
            this.inhaltChanged = true;
            this.updateActionButtons();
        }
    }

    private void checkRegisterSize(final RegisterTypEnum typ) throws RemoteException {

        this.checkRegisterSize(typ.getIndex().length, typ);
    }

    private void handleCancelEdit() {

        this.inhaltChanged = false;
        this.updateActionButtons();
        GUI.startView(OrdnerListView.class.getName(), null);
    }

    /**
     * Saves the folder and register
     *
     * @throws ApplicationException
     */
    private void handleSave() throws ApplicationException {

        Logger.debug("Save folder..."); //$NON-NLS-1$
        // Erst mal den Ordner speichern
        try {
            this.ordner.setWidth(this.ordnerRuecken.getWidth());
            this.ordner.setTitel(this.getOrdnerRuecken().getTitelText());
            this.ordner.setTitelFont(this.getOrdnerRuecken().getFont(LabelPartSectionsEnum.TITLE));
            this.ordner.setTitelColor(this.getOrdnerRuecken().getFgColour(LabelPartSectionsEnum.TITLE));
            this.ordner.setTitelAlignment(this.getOrdnerRuecken().getAlignment(LabelPartSectionsEnum.TITLE));
            this.ordner.setContent(this.getOrdnerRuecken().getContentText());
            this.ordner.setContentFont(this.getOrdnerRuecken().getFont(LabelPartSectionsEnum.CONTENT));
            this.ordner.setContentColor(this.getOrdnerRuecken().getFgColour(LabelPartSectionsEnum.CONTENT));
            this.ordner.setContentAlignment(this.getOrdnerRuecken().getAlignment(LabelPartSectionsEnum.CONTENT));
            this.ordner.setFooter(this.getOrdnerRuecken().getFooterText());
            this.ordner.setFooterFont(this.getOrdnerRuecken().getFont(LabelPartSectionsEnum.FOOTER));
            this.ordner.setFooterColor(this.getOrdnerRuecken().getFgColour(LabelPartSectionsEnum.FOOTER));
            this.ordner.setFooterAlignment(this.getOrdnerRuecken().getAlignment(LabelPartSectionsEnum.FOOTER));
            this.ordner.setIdVisible(this.getOrdnerRuecken().isIdVisible());
            this.ordner.setLabelBackground(this.getOrdnerRuecken().getBgColour());
            this.ordner.setBarColor(this.getOrdnerRuecken().getBarColour());
            this.ordner.setBarVisible(this.getOrdnerRuecken().getBarVisible());
            this.ordner.setRegisterIndexFont(this.getRegisterSettingsPart().getIndexFont());
            this.ordner.setRegisterIndexBackground(this.getRegisterSettingsPart().getIndexBgColor());
            this.ordner.setRegisterContentFont(this.getRegisterSettingsPart().getContentFont());
            this.ordner.setRegisterContentBackground(this.getRegisterSettingsPart().getContentBgColor());
            this.ordner.store();
            // Speichern vom Register
            final ArrayList<String> registerIds = new ArrayList<>();
            final List<?> regList = this.registerTable.getItems();
            int position = 0;
            for (final Object object : regList) {
                final Register registerEintrag = (Register) object;
                registerEintrag.setOrdner(this.ordner);
                registerEintrag.setPosition(position++);
                registerEintrag.store();
                registerIds.add(registerEintrag.getID());
            }
            // Entfernen von gelschten Registern
            final DBIterator<Register> listRegister = this.ordner.getInhaltsverzeichniss();
            while (listRegister.hasNext()) {
                final Register registerEintrag = listRegister.next();
                if (!registerIds.contains(registerEintrag.getID())) {
                    registerEintrag.delete();
                }
            }
            this.neuerOrdner = false;
            this.inhaltChanged = false;
            this.updateActionButtons();
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("OrdnerBearbeitenController.SaveFolderFailed"), e); //$NON-NLS-1$
        }
    }

    private void updateActionButtons() {

        if (this.inhaltChanged) {
            this.getSaveButton().setEnabled(true);
            this.getPrintButton().setEnabled(false);
        } else {
            this.getSaveButton().setEnabled(false);
            this.getPrintButton().setEnabled(true);
        }
    }
}
