/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.gui.controller;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.gui.input.TextStyle;
import de.frankmuenster.jameica.orverw.gui.input.TextStyleInput;
import de.frankmuenster.jameica.orverw.gui.views.SettingsView;
import de.frankmuenster.jameica.orverw.parts.HorizontalLinePart;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.util.ApplicationException;

/**
 * Kontroller fr die Settings View
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class SettingsController extends AbstractControl {

    /**
     * Das Part f&uuml;r die obere Linie
     */
    private transient HorizontalLinePart barUpper;

    /**
     * Das Part f&uuml;r die untere Linie
     */
    private transient HorizontalLinePart barLower;

    /**
     * Die Farbe der Linien
     */
    private transient Color barColor;

    /**
     * Der Button f&uuml; die Linienfarbe
     */
    private transient Button barColorButton;

    /**
     * Die Sichbarkeitscheckbox der Linien
     */
    private transient CheckboxInput barVisible;

    /**
     * Der Texttyp des Inhaltbereiches
     */
    private transient TextStyleInput contentStyle;

    /**
     * Der TZexttype des Fu&szlig;bereiches
     */
    private transient TextStyleInput footerStyle;

    /**
     * Beipsieltext des ID Feldes
     */
    private transient TextInput idExample;

    /**
     * Checkbox ID Sichtbar
     */
    private transient CheckboxInput idVisible;

    /**
     * Hintergrundfarbe des Labels
     */
    private transient Color lblBgColor;

    /**
     * Buttons f&uuml; die Hintergrundfarbe des Labels
     */
    private transient Button lblBgButton;

    /**
     * Texttyp desd Inhaltbereiches
     */
    private transient TextStyleInput regContentStyle;

    /**
     * Texttyp des Registerindex
     */
    private transient TextStyleInput regIndexStyle;

    /**
     * Texttyp des Titelbereiches
     */
    private transient TextStyleInput titleSytle;

    /**
     * Erstellt eine neue Instanz des Kontrollers
     *
     * @param view
     */
    public SettingsController(final SettingsView view) {

        super(view);
        this.view = view;

    }

    /**
     * Button f&uuml;r die Bar-Hintergrundfarbe
     *
     * @return a Button to set the colour of the bars
     */
    public Button getBarColorButton() {

        if (this.barColorButton == null) {

            this.barColor = Settings.getDefaultBarColor();
            this.barColorButton = new Button(I18N.tr("SettingsController.lineColour"), new Action() { //$NON-NLS-1$

                /**
                 * @see Action#handleAction(Object)
                 */
                @Override
                public void handleAction(final Object context) throws ApplicationException {

                    final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
                    colorDialog.setRGB(SettingsController.this.barColor.getRGB());
                    colorDialog.setText(I18N.tr("SettingsController.chooseColour")); //$NON-NLS-1$
                    final RGB colour = colorDialog.open();
                    if (colour != null) {
                        SettingsController.this.barColor = new Color(GUI.getDisplay(), colour);
                        // Update bar colours
                        SettingsController.this.updateBarColours();
                    }
                }
            });

            // Update demo lines colours
            this.updateBarColours();

        }
        return this.barColorButton;
    }

    /**
     * @return die die HorizontalLinePart fr die untere DemoLinie.
     */
    public HorizontalLinePart getBarLower() {

        if (this.barLower == null) {

            final Point size = new Point(200, 8);
            this.barLower = new HorizontalLinePart(this.barColor, size, false);

            if (this.barVisible != null) {
                this.barLower.setVisible((boolean) this.barVisible.getValue());
            }
        }
        return this.barLower;
    }

    /**
     * @return die HorizontalLinePart fr die obere DemoLinie.
     */
    public HorizontalLinePart getBarUpper() {

        if (this.barUpper == null) {

            final Point size = new Point(200, 8);
            this.barUpper = new HorizontalLinePart(this.barColor, size, false);

            if (this.barVisible != null) {
                this.barUpper.setVisible((boolean) this.barVisible.getValue());
            }

        }
        return this.barUpper;
    }

    /**
     * @return Ein CheckBoxInput fr Trennlinien-Sichtbarkeit.
     */
    public CheckboxInput getBarVisible() {

        if (this.barVisible == null) {

            this.barVisible = new CheckboxInput(Settings.isDefaultBarVisible());
            this.barVisible.setName(I18N.tr("SettingsController.showLines")); //$NON-NLS-1$

            this.barVisible.addListener(new Listener() {

                /**
                 * @see Action#handleAction(Object)
                 */
                @Override
                public void handleEvent(final Event arg0) {

                    // Set visibility of demo lines
                    if (SettingsController.this.barUpper != null) {
                        SettingsController.this.barUpper
                                .setVisible((boolean) SettingsController.this.barVisible.getValue());
                    }
                    if (SettingsController.this.barLower != null) {
                        SettingsController.this.barLower
                                .setVisible((boolean) SettingsController.this.barVisible.getValue());
                    }
                }
            });

            // Set visibility of demo lines
            if (this.barUpper != null) {
                this.barUpper.setVisible((boolean) this.barVisible.getValue());
            }
            if (this.barLower != null) {
                this.barLower.setVisible((boolean) this.barVisible.getValue());
            }

        }
        return this.barVisible;
    }

    /**
     * @return TextStyleInput f&uuml; den Inhaltsbereich.
     * @throws ApplicationException
     */
    public TextStyleInput getContentStyle() throws ApplicationException {

        if (this.contentStyle == null) {

            final TextStyle content = Settings.getDefaultContentStyle();
            content.setBackgroundColor(this.lblBgColor);
            this.contentStyle = new TextStyleInput(content, TextStyleInput.ALIGNMENT | TextStyleInput.FOREGROUND);
        }
        return this.contentStyle;
    }

    /**
     * @return TextStyleInput f&uuml;r den Fussbereich.
     * @throws ApplicationException
     */
    public TextStyleInput getFooterStyle() throws ApplicationException {

        if (this.footerStyle == null) {

            final TextStyle footer = Settings.getDefaultFooterStyle();
            footer.setBackgroundColor(this.lblBgColor);
            this.footerStyle = new TextStyleInput(footer, TextStyleInput.ALIGNMENT | TextStyleInput.FOREGROUND);
        }
        return this.footerStyle;
    }

    /**
     * @return Input f&uuml;r das ID-Beispiel
     */
    public TextInput getIdExample() {

        if (this.idExample == null) {

            this.idExample = new TextInput("99"); //$NON-NLS-1$
            this.idExample.setEnabled(false);

        }
        return this.idExample;

    }

    /**
     * @return Input fr die ID-Sichtbarkeit
     */
    public CheckboxInput getIdVisible() {

        if (this.idVisible == null) {

            this.idVisible = new CheckboxInput(Settings.isDefaultIdVisible());
            this.idVisible.setName(I18N.tr("SettingsController.showId")); //$NON-NLS-1$
            this.idVisible.addListener(new Listener() {

                /**
                 * @see Action#handleAction(Object)
                 */
                @Override
                public void handleEvent(final Event arg0) {

                    if (SettingsController.this.idExample != null) {
                        SettingsController.this.idExample.getControl()
                                .setVisible((Boolean) SettingsController.this.idVisible.getValue());
                    }
                }
            });

        }
        return this.idVisible;
    }

    /**
     * Button f&uuml;r die Label-Hintergrundfarbe
     *
     * @return a button to set the background colour of the label
     */
    public Button getLabelBackgroundButton() {

        if (this.lblBgButton == null) {

            this.lblBgColor = Settings.getDefaultLabelBackground();
            this.lblBgButton = new Button(I18N.tr("SettingsController.labelBgColour"), new LabelBgButtonAction()); //$NON-NLS-1$

        }
        return this.lblBgButton;
    }

    /**
     * @return TextStyleInput f&uuml;r den Register-Inhalt
     * @throws ApplicationException
     */
    public TextStyleInput getRegisterContentStyle() throws ApplicationException {

        if (this.regContentStyle == null) {

            this.regContentStyle = new TextStyleInput(Settings.getDefaultRegisterContentStyle(),
                    TextStyleInput.BACKGROUND | TextStyleInput.FOREGROUND, new Point(100, 0));
        }
        return this.regContentStyle;
    }

    /**
     * @return TextStyleInput f&uuml;r den Register-Index
     * @throws ApplicationException
     */
    public TextStyleInput getRegisterIndexStyle() throws ApplicationException {

        if (this.regIndexStyle == null) {

            this.regIndexStyle = new TextStyleInput(Settings.getDefaultRegisterIndexStyle(),
                    TextStyleInput.BACKGROUND | TextStyleInput.FOREGROUND, new Point(100, 0));
        }
        return this.regIndexStyle;
    }

    /**
     * @return Ein TextStyleInput fr den Titel-Style.
     * @throws ApplicationException
     */
    public TextStyleInput getTitleStyle() throws ApplicationException {

        if (this.titleSytle == null) {

            final TextStyle title = Settings.getDefaultTitleStyle();
            title.setBackgroundColor(this.lblBgColor);
            this.titleSytle = new TextStyleInput(title, TextStyleInput.ALIGNMENT | TextStyleInput.FOREGROUND);
        }
        return this.titleSytle;
    }

    /**
     * Speichert die aktuellen Einstellungen im Settings Objekt.
     */
    public void speichern() {

        Settings.setDefaultBarColor(this.barColor);
        Settings.setDefaultBarVisible((Boolean) this.barVisible.getValue());
        Settings.setDefaultContentStyle((TextStyle) this.contentStyle.getValue());
        Settings.setDefaultFooterStyle((TextStyle) this.footerStyle.getValue());
        Settings.setDefaultIdVisible((Boolean) this.idVisible.getValue());
        Settings.setDefaultLabelBackground(this.lblBgColor);
        Settings.setDefaultRegisterContentStyle((TextStyle) this.regContentStyle.getValue());
        Settings.setDefaultRegisterIndexStyle((TextStyle) this.regIndexStyle.getValue());
        Settings.setDefaultTitleStyle((TextStyle) this.titleSytle.getValue());

        GUI.getStatusBar().setSuccessText(I18N.tr("SettingsController.settingsSaved")); //$NON-NLS-1$

    }

    private void updateBarColours() {

        if (this.barUpper != null) {
            this.barUpper.setColor(this.barColor);
        }
        if (this.barLower != null) {
            this.barLower.setColor(this.barColor);
        }

    }

    /**
     * Action to handle the label background button
     *
     * @author Frank Mnster
     * @since 1.0
     */
    private class LabelBgButtonAction implements Action {

        /**
         * Creates a new instance to handle the label background button actions
         */
        public LabelBgButtonAction() {

            // Absichtliche leer
        }

        /**
         * @see Action#handleAction(Object)
         */
        @Override
        public void handleAction(final Object context) throws ApplicationException {

            final ColorDialog colorDialog = new ColorDialog(GUI.getShell());
            colorDialog.setRGB(SettingsController.this.lblBgColor.getRGB());
            colorDialog.setText(I18N.tr("SettingsController.chooseColour")); //$NON-NLS-1$
            final RGB colour = colorDialog.open();
            if (colour != null) {
                SettingsController.this.lblBgColor = new Color(GUI.getDisplay(), colour);

                // Update example text background colours
                if (SettingsController.this.titleSytle != null) {
                    final TextStyle style = (TextStyle) SettingsController.this.titleSytle.getValue();
                    style.setBackgroundColor(SettingsController.this.lblBgColor);
                    SettingsController.this.titleSytle.setValue(style);
                }
                if (SettingsController.this.contentStyle != null) {
                    final TextStyle style = (TextStyle) SettingsController.this.contentStyle.getValue();
                    style.setBackgroundColor(SettingsController.this.lblBgColor);
                    SettingsController.this.contentStyle.setValue(style);
                }
                if (SettingsController.this.footerStyle != null) {
                    final TextStyle style = (TextStyle) SettingsController.this.footerStyle.getValue();
                    style.setBackgroundColor(SettingsController.this.lblBgColor);
                    SettingsController.this.footerStyle.setValue(style);
                }
            }

        }

    }

}
