/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.gui.views;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.layout.GridLayout;

import de.frankmuenster.jameica.orverw.gui.action.ViewOrdnerVerwaltungAction;
import de.frankmuenster.jameica.orverw.gui.controller.SettingsController;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.util.ApplicationException;

/**
 * View fr die Einstellungen
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class SettingsView extends AbstractView {

    /**
     * Unser Standartkonstruktor
     */
    public SettingsView() {

        super();
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws RemoteException, ApplicationException {

        GUI.getView().setTitle(I18N.tr("SettingsView.viewTitle")); //$NON-NLS-1$
        final SettingsController control = new SettingsController(this);

        final SimpleContainer container = new SimpleContainer(this.getParent(), true, 1);

        final GridLayout gridLayout = new GridLayout(2, false);
        gridLayout.verticalSpacing = 10;

        final ColumnLayout columnLayout = new ColumnLayout(container.getComposite(), 2);
        final SimpleContainer left = new SimpleContainer(columnLayout.getComposite(), true);
        final SimpleContainer right = new SimpleContainer(columnLayout.getComposite(), false, 1);

        left.addHeadline(I18N.tr("SettingsView.haedlineLabel")); //$NON-NLS-1$
        final SimpleContainer genSet = new SimpleContainer(left.getComposite(), false, 3);
        control.getLabelBackgroundButton().paint(genSet.getComposite());
        control.getBarColorButton().paint(genSet.getComposite());
        control.getBarVisible().paint(genSet.getComposite());

        left.addText(StringUtils.EMPTY, false);
        left.addPart(control.getBarUpper());
        left.addHeadline(I18N.tr("SettingsView.headLineTitleStyle")); //$NON-NLS-1$
        left.addPart(control.getTitleStyle());
        left.addPart(control.getBarLower());
        left.addHeadline(I18N.tr("SettingsView.headlineContentStyle")); //$NON-NLS-1$
        left.addPart(control.getContentStyle());
        left.addHeadline(I18N.tr("SettingsView.headlineFooterStyle")); //$NON-NLS-1$
        left.addPart(control.getFooterStyle());
        left.addSeparator();
        final SimpleContainer idExample = new SimpleContainer(left.getComposite(), false, 4);
        idExample.addLabelPair(control.getIdExample(), control.getIdVisible());
        control.getIdExample().getControl().setVisible((boolean) control.getIdVisible().getValue());

        right.addHeadline(I18N.tr("SettingsView.headLineRegisterSettings")); //$NON-NLS-1$
        right.addText(StringUtils.EMPTY, false);
        right.addHeadline(I18N.tr("SettingsView.headlineIndex")); //$NON-NLS-1$
        right.addPart(control.getRegisterIndexStyle());
        right.addText(StringUtils.EMPTY, false);
        right.addHeadline(I18N.tr("SettingsView.headlineDescription")); //$NON-NLS-1$
        right.addPart(control.getRegisterContentStyle());

        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(I18N.tr("_save"),  //$NON-NLS-1$
                (final Object context) -> control.speichern(), null, true, "document-save.png"); //$NON-NLS-1$
        buttons.addButton(I18N.tr("_cancel"), new ViewOrdnerVerwaltungAction()); //$NON-NLS-1$

        container.addText(StringUtils.EMPTY, false);
        container.addButtonArea(buttons);

    }

}
