/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.orverw.parts.labelpart;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;

/**
 * A button to select a font or colour for a given section
 *
 * @author Frank Mnster
 * @since 1.0
 */
public final class LabelPartButtonObject {

    /**
     * Bereich f&uuml;r den die Objektdaten gelten
     */
    private LabelPartSectionsEnum section;

    /**
     * Button der ausgel&ouml;st hat
     */
    private LabelPartButtonsEnum button;

    /**
     * Der ausgew&auml;hlte Schrifttyp
     */
    private Font font;
    /**
     * Die ausgew&auml;hlte Farbe
     */
    private Color colour;

    /**
     * Konstruktor mit allen Feldern. Es sind nicht alle Felder obligatorisch.
     * So kann der Schrifttyp bzw. die Farbe auch null sein.
     *
     * @param section
     *            der Bereich f&uuml;r den die Objektdaten gelten
     * @param button
     *            der Button der ausgel&ouml;st hat
     * @param font
     *            der ausgew&auml;hlte Schrifttyp
     * @param color
     *            die ausgew&auml;hlte Farbe
     */
    public LabelPartButtonObject(final LabelPartSectionsEnum section, final LabelPartButtonsEnum button,
	    final Font font, final Color color) {
	this.section = section;
	this.button = button;
	this.font = font;
	this.colour = color;
    }

    /**
     * Liefert den Button der ausgel&ouml;st hat
     *
     * @return button Der Button der ausgel&ouml;st hat
     */
    public LabelPartButtonsEnum getButton() {
	return this.button;
    }

    /**
     * Liefert die ausgew&auml;hlte Farbe
     *
     * @return colour Die ausgew&auml;hlte Farbe
     */
    public Color getColour() {
	return this.colour;
    }

    /**
     * Liefert den ausgew&auml;hlte Schrifttyp
     *
     * @return font Den ausgew&auml;hlte Schrifttyp
     */
    public Font getFont() {
	return this.font;
    }

    /**
     * Liefert den Bereich f&uuml;r den die Objektdaten gelten
     *
     * @return section Der Bereich f&uuml;r den die Objektdaten gelten
     */
    public LabelPartSectionsEnum getSection() {
	return this.section;
    }

    /**
     * Setzt den Button der ausgel&ouml;st hat
     *
     * @param button
     *            der Button der ausgel&ouml;st hat
     */
    public void setButton(final LabelPartButtonsEnum button) {
	this.button = button;
    }

    /**
     * Setzt die ausgew&auml;hlte Farbe
     *
     * @param colour
     *            die ausgew&auml;hlte Farbe
     */
    public void setColour(final Color colour) {
	this.colour = colour;
    }

    /**
     * Setzt den ausgew&auml;hlte Schrifttyp
     *
     * @param font
     *            der ausgew&auml;hlte Schrifttyp
     */
    public void setFont(final Font font) {
	this.font = font;
    }

    /**
     * Setzt den Bereich f&uuml;r den die Objektdaten gelten
     *
     * @param section
     *            der Bereich f&uuml;r den die Objektdaten gelten
     *            (LabelPartSectionsEnum)
     */
    public void setSection(final LabelPartSectionsEnum section) {
	this.section = section;
    }

}
