/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.parts.register;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

/**
 * Bean-Objekt mit angeben &uuml;ber einen gedr&uuml;ckten Register-Einstellungs-Button, die ausgew&auml;hlte Schrift
 * und/oder die ausgew&auml;hlte Farbe.
 *
 * @author Frank Mnster
 * @since 1.0
 */
public class RegisterSettingsButtonObject {

    /**
     * Der Register-Einstellungs-Button
     */
    private RegisterSettingsButtonsEnum button;

    /**
     * Die ausge&auml;hlte Schrift
     */
    private Font font;

    /**
     * Die ausge&auml;hlte Farbe
     */
    private Color color;

    /**
     * Konstruktor mit Button und Farbe
     * 
     * @param button
     *            the type of the button
     * @param color
     *            the colour of the button text
     */
    public RegisterSettingsButtonObject(final RegisterSettingsButtonsEnum button, final Color color) {

        this(button, null, color);
    }

    /**
     * Konstruktor mit Button und Schrift
     * 
     * @param button
     *            the type of the button
     * @param font
     *            the font for the button
     */
    public RegisterSettingsButtonObject(final RegisterSettingsButtonsEnum button, final Font font) {

        this(button, font, null);
    }

    /**
     * Konstruktor mit allen Feldern
     * 
     * @param button
     *            the type of the button
     * @param font
     *            the font for the button
     * @param color
     *            the colour of the button text
     */
    public RegisterSettingsButtonObject(final RegisterSettingsButtonsEnum button, final Font font, final Color color) {

        this.button = button;
        this.font = font;
        this.color = color;
    }

    /**
     * @return the button
     */
    public RegisterSettingsButtonsEnum getButton() {

        return this.button;
    }

    /**
     * @return the colour
     */
    public Color getColor() {

        return this.color;
    }

    /**
     * @return the font
     */
    public Font getFont() {

        return this.font;
    }

    /**
     * @param button
     *            the button to set
     */
    public void setButton(final RegisterSettingsButtonsEnum button) {

        this.button = button;
    }

    /**
     * @param color
     *            the colour to set
     */
    public void setColor(final Color color) {

        this.color = color;
    }

    /**
     * @param font
     *            the font to set
     */
    public void setFont(final Font font) {

        this.font = font;
    }

}
