/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.nebula.paperclips.core.EmptyPrint;
import org.eclipse.nebula.paperclips.core.Margins;
import org.eclipse.nebula.paperclips.core.PaperClips;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.PrintJob;
import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.widgets.PrintViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseWheelListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.ScrolledContainer;
import de.willuhn.util.ApplicationException;

/**
 * Zeigt eine einfache Druckvorschau. Ist noch verbesserungswrdig. Benutzt das PrintViewer Objekt aus paperclips, da
 * PrintPreview in Linux noch buggy ist.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.BeanMembersShouldSerialize" })
public class PrintViewDialog {

    /**
     * Die Shell fr den Dialog
     */
    private Shell shell;

    /**
     * Ein Scrolled Container, damit auch alles Sichtbar ist
     */
    private ScrolledContainer scroll;

    /**
     * Das PrintViewer Objekt.
     */
    private PrintViewer printview;

    /**
     * Der Seitentitel, kommt aus dem Ordner Titelfeld
     */
    private final String titel;

    /**
     * Das Page Objekt zur Druckvorschau
     */
    private final Print page;

    /**
     * Die Hauptanwendnung zu dem Dialog.
     */
    private final Shell parentShell;

    /**
     * Der Konstruktor erstellt schon mal alles, zeigt aber nix an.
     *
     * @param print
     *            das Print-Objekt das angezeigt werden soll.
     * @param parentShell
     *            die Hauptanwendnung zu dem Dialog
     * @throws ApplicationException
     */
    public PrintViewDialog(final AbstractPrint print, final Shell parentShell) throws ApplicationException {

        this.titel = print.getTitle();
        this.page = print.createPrint();
        this.parentShell = parentShell;
    }

    /**
     * &Ouml;ffent die Druckvorschau in einem Dialogfenster.
     *
     * @return Die Shell f&uuml; den Dialog.
     * @throws ApplicationException
     */
    public Shell open() throws ApplicationException {

        this.shell = new Shell(this.parentShell, SWT.SHELL_TRIM | SWT.RESIZE | SWT.APPLICATION_MODAL);
        this.shell.setText(I18N.tr("PrintViewDialog.printPreview")); //$NON-NLS-1$
        this.shell.setBounds(100, 100, 830, 630);
        this.shell.setLayout(new GridLayout(1, false));
        try {
            this.createButtonPanel(this.shell);
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintViewDialog.errorOpenPrintPreview"), e); //$NON-NLS-1$
        }
        this.createPrinterView(this.shell);
        if (this.page instanceof PagePrint) {
            final PagePrint pagePrint = (PagePrint) this.page;
            // Ein wenig Abstand am Ende einfgen
            if (pagePrint.getBody() instanceof GridPrint) {
                final GridPrint body = (GridPrint) pagePrint.getBody();
                final int width = body.getColumns()[0].size;
                final EmptyPrint topP = new EmptyPrint(width, 20);
                body.add(topP);
            }
        }
        this.printview.setPrint(this.page);
        this.shell.setVisible(true);
        while (!this.shell.isDisposed()) {
            if (!this.shell.getDisplay().readAndDispatch()) {
                this.shell.getDisplay().sleep();
            }
        }
        return this.shell;
    }

    /**
     * Hier wird gedruckt
     */
    protected void handlePrint() {

        final PrintDialog dialog = new PrintDialog(this.shell, SWT.NONE);
        final PrinterData printerData = dialog.open();
        if (printerData != null) {
            // Erstelle Printjob
            final PrintJob job = new PrintJob(I18N.tr("PrintViewDialog.printJobTitlePrefix") + this.titel, //$NON-NLS-1$
                    this.page);
            final Margins margins = job.getMargins();
            margins.left = 100;
            margins.right = 50;
            // Drucken starten
            PaperClips.print(job, printerData);
            // ...und wir schlieen die Vorschau
            this.shell.dispose();
        }
    }

    /**
     * Erstellt die Button zum drucken, bzw. Abbrechen
     *
     * @param parent
     *            Wo die Button gemalt werden sollen
     * @throws RemoteException
     */
    private void createButtonPanel(final Composite parent) throws RemoteException {

        final Button print = new Button(I18N.tr("_print"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                // Das Drucken behandeln wir in eine extra Funktion
                PrintViewDialog.this.handlePrint();
            }
        }, null, false, "document-print.png"); //$NON-NLS-1$
        final Button cancel = new Button(I18N.tr("_close"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                // Das Drucken behandeln wir in eine extra Funktion
                PrintViewDialog.this.shell.dispose();
            }
        }, null, false, "window-close.png"); //$NON-NLS-1$
        final ButtonArea buttonArea = new ButtonArea();
        buttonArea.addButton(print);
        buttonArea.addButton(cancel);
        buttonArea.paint(parent);
    }

    /**
     * Erstellt und liefert die eigentliche Druckvorschau
     *
     * @param parent
     *            in der die Druckvorschau gemalt werden soll
     * @return the control showing the print preview
     */
    private Control createPrinterView(final Composite parent) {

        this.scroll = new ScrolledContainer(parent, 3);
        // Oberer Rand mit Hinweistext
        final Label hint = new Label(this.scroll.getComposite(), SWT.NONE);
        final GridData gd1 = new GridData();
        gd1.heightHint = (int) (12 * 0.039 * 96);
        gd1.horizontalSpan = 3;
        hint.setLayoutData(gd1);
        hint.setText(I18N.tr("PrintViewDialog.printHint")); //$NON-NLS-1$
        // linker Rand
        final Label left = new Label(this.scroll.getComposite(), SWT.NONE);
        left.setText(StringUtils.EMPTY);
        GridData gridData = new GridData();
        gridData.widthHint = (int) (8 * 0.039 * 96);
        left.setLayoutData(gridData);
        // Container fr den PrintViewer, damit die Gre halbwegs passt
        final Composite container = new Composite(this.scroll.getComposite(), SWT.BORDER);
        final GridLayout layout = new GridLayout(1, false);
        container.setLayout(layout);
        container.setBackground(new Color(parent.getDisplay(), 255, 255, 255));
        // rechter Rand
        final Label right = new Label(this.scroll.getComposite(), SWT.NONE);
        right.setText(StringUtils.EMPTY);
        gridData = new GridData();
        gridData.widthHint = (int) (8 * 0.039 * 96);
        right.setLayoutData(gridData);
        // Linken Rand, damit es schne raussieht
        final Label linkerRand = new Label(container, SWT.NONE);
        gridData = new GridData();
        gridData.heightHint = (int) (10 * 0.039 * 96);
        linkerRand.setLayoutData(gridData);
        linkerRand.setText(StringUtils.EMPTY);
        // Grob ein A4 Blatt mit 2 cm Rand
        gridData = new GridData();
        gridData.widthHint = (int) (160 * 0.039 * 96);
        gridData.heightHint = (int) (290 * 0.039 * 96);
        gridData.horizontalIndent = (int) (20 * 0.039 * 96);
        this.printview = new PrintViewer(container, SWT.NONE);
        this.printview.getControl().setLayoutData(gridData);
        // Eigener Listener fr den PrintViewer, da er ja selbst ein
        // ScrolledComposit ist
        this.printview.getControl().addMouseWheelListener(new MyMouseWheelListener());
        // scroll Container an Gre anpassen
        this.scroll.update();
        return this.scroll.getComposite();
    }

    /**
     * Unser privater MouseWheelListener
     *
     * @author Frank Mnster
     */
    private class MyMouseWheelListener implements MouseWheelListener {

        /**
         * Die Bildschirmauflsung
         */
        private final transient Point dpi = Display.getDefault().getDPI();

        /**
         * Returns a new MouseWheelListener to handle scrolling with the mouse wheel.
         */
        public MyMouseWheelListener() {

            // Intentionally left blank
        }

        /**
         * @see org.eclipse.swt.events.MouseWheelListener#mouseScrolled(org.eclipse.swt.events.MouseEvent)
         */
        @Override
        public void mouseScrolled(final MouseEvent event) {

            final ScrolledComposite scrolledComposite = (ScrolledComposite) PrintViewDialog.this.scroll.getComposite()
                    .getParent();
            final Rectangle bounds = scrolledComposite.getClientArea();
            final Point origin = scrolledComposite.getOrigin();
            final int direction = event.count > 0 ? -1 : 1;
            final Point size = PrintViewDialog.this.printview.getControl().getSize();
            if (size.y > bounds.height && event.stateMask == SWT.NONE) {
                origin.y += direction * Math.min(this.dpi.y, bounds.height / 4);
            } else if (size.x > bounds.width) {
                origin.x += direction * Math.min(this.dpi.x, bounds.width / 4);
            }
            scrolledComposite.setOrigin(origin);
        }
    }
}
